//=============================================================================
//              |
// Headerfile   |  ftMscLib.h
//              |
// Description  |  Function prototypes of Library ftMscLib
//              |
//-----------------------------------------------------------------------------
// Disclaimer - Exclusion of Liability
//
// This software is distributed in the hope that it will be useful,but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. It can be used an modified by anyone
// free of any license obligations or authoring rights.
//=============================================================================

#ifndef __FTMSCLIB_H__
#define __FTMSCLIB_H__


#ifndef  __cplusplus
	typedef BOOL bool;
#endif

#if defined(_MSC_VER)
#define DLLImp extern "C" __declspec(dllimport)
#else
#define DLL
#endif

//  port restriction COM ports
enum PortSelect {
    ALL_PORTS = 0,
    USB_ONLY,
    BT_ONLY
};


//  Fish.X1 TransferArea running status
//      0 - TransferAera is not running
//      1 - TransferArea is running
//      2 - TransferArea is synchronized
#define FX1STOP     0
#define FX1RUN      1
#define FX1SYNC     2


//  mult-interface-mode status (slave)
#define SLAVE_OFFLINE       0
#define SLAVE_ONLINE        1



// ============================================================================
//  TransferArea of all ROBO Interfaces (Master + 8 Slaves)
//-----------------------------------------------------------------------------


//  TransferArea of ftMscLib (Master IF, 8 Slave IF) 
typedef struct _if08_transfer {

	TA	ftxTransferArea[TA_N_PARTS];

} IF08_TRANSFER;



//  ftMscLib import functions
//-----------------------------------------------------------------------------
DLLImp DWORD ftxGetLibVersion(void);
DLLImp DWORD ftxGetLibVersionStr(LPSTR, DWORD);
DLLImp DWORD ftxInitLib(void);
DLLImp DWORD ftxCloseLib(void);
DLLImp DWORD ftxIsLibInit(void);
DLLImp DWORD ftxGetSerialNrStrg (HANDLE, LPSTR, DWORD);
DLLImp DWORD ftxGetSerialNr (HANDLE);
DLLImp DWORD ftxGetFirmwareStrg(HANDLE, LPSTR, DWORD);
DLLImp DWORD ftxGetFirmware(HANDLE);
DLLImp DWORD ftxGetManufacturerStrg(HANDLE, LPSTR, DWORD);
DLLImp DWORD ftxGetShortNameStrg(HANDLE, LPSTR, DWORD);
DLLImp DWORD ftxGetLongNameStrg(HANDLE, LPSTR, DWORD);
DLLImp DWORD ftxGetLibErrorString(DWORD, DWORD, LPSTR, DWORD);

//  open, close connection to ROBO interface
DLLImp HANDLE ftxOpenComDevice(char *, DWORD, DWORD *);
DLLImp HANDLE ftxOpenComDeviceNr(DWORD, DWORD, DWORD *);
DLLImp DWORD ftxCloseDevice(HANDLE);
DLLImp DWORD ftxCloseAllDevices(void);

//  start, stop, status TransferArea
DLLImp DWORD ftxStartTransferArea(HANDLE);
DLLImp DWORD ftxStopTransferArea(HANDLE);
DLLImp DWORD ftxIsTransferActiv(HANDLE);

//  get ftX1 Transfer address
DLLImp volatile IF08_TRANSFER *GetIF08TransferAddress(HANDLE);

//  communication status and info
DLLImp DWORD ftxIsHandleValid(HANDLE);
DLLImp DWORD GetComStatus(HANDLE);
DLLImp DWORD GetAvailableComPorts(int);
DLLImp DWORD EnumComPorts(DWORD, LPSTR, DWORD);

//  set fish.X1 output structure
DLLImp DWORD SetOutCounterReset(HANDLE, int, int);
DLLImp DWORD SetOutMotorValues(HANDLE, int, int, int, int, bool);
DLLImp DWORD SetOutPwmValues(HANDLE, int, int, int);

//  set fish.X1 info structure
DLLImp DWORD SetRoboTxDevName(HANDLE, int, LPSTR);
DLLImp DWORD SetRoboTxMessage(HANDLE, int, LPCSTR);

//  set fish.X1 config structure
DLLImp DWORD SetFtUniConfig (HANDLE, int, int, int, bool);
DLLImp DWORD SetFtCntConfig (HANDLE, int, int, int);
DLLImp DWORD SetFtMotorConfig (HANDLE, int, int, bool);
DLLImp DWORD SetMotorExConfig(HANDLE, int, int, int, int, int, int);
DLLImp DWORD StopMotorExConfig(HANDLE, int);
DLLImp DWORD ResetMotorExConfig(HANDLE, int, int);
DLLImp void  SetCBMotorExReached(void (__stdcall *)(DWORD, DWORD));

//  get values from fish.X1 TransferArea, input structure
DLLImp DWORD GetInIOValue(HANDLE, int, int, INT16 *, BOOL32 *);
DLLImp DWORD GetInCounterValue (HANDLE, int, int, INT16 *, INT16 *);
DLLImp DWORD GetInDisplayButtonValue (HANDLE, int, INT16 *, INT16 *);

//  file upload and remote commands to Robo08 Interface
DLLImp DWORD FtRemoteCmd(HANDLE, char *, void (__stdcall *)(LPSTR, DWORD));
DLLImp DWORD FtFileUpload(HANDLE, char *, DWORD, void (__stdcall *)(DWORD));
DLLImp DWORD FtRamFileUpload(HANDLE, DWORD, CONST PVOID, DWORD, LPCSTR, void (__stdcall *)(DWORD));
DLLImp DWORD RTxCleanDisk(HANDLE, DWORD);

//  set local program status
DLLImp DWORD FtProgramRun(HANDLE, int);
DLLImp DWORD FtProgramStop(HANDLE, int);

//  for Multi-Interface-Mode
DLLImp DWORD GetIFMasterState(HANDLE);
DLLImp DWORD GetIFSlaveAlive(HANDLE, int, BOOL *);
DLLImp DWORD GetIF08Info(HANDLE, int);
DLLImp DWORD GetIFSlaveState(HANDLE, BYTE *);

//  fish.X1 info structure
DLLImp DWORD GetRoboTxDevName(HANDLE, int, LPSTR, DWORD);
DLLImp DWORD GetRoboTxBtAddr(HANDLE, int, LPSTR, DWORD);
DLLImp DWORD GetRoboTxFwStr(HANDLE, int, LPSTR, DWORD);
DLLImp DWORD GetRoboTxFwVal(HANDLE, int, DWORD *);
DLLImp DWORD GetRoboTxHwStr(HANDLE, int, LPSTR, DWORD);
DLLImp DWORD GetRoboTxMemLayout(HANDLE, int, PULONG, PULONG, PULONG);
DLLImp DWORD GetRoboTxSerialStr(HANDLE, int, LPSTR, DWORD);
DLLImp DWORD GetRoboTxDllStr(HANDLE, int, LPSTR, DWORD);

//  set callback function for Multi-Interface-Mode status
DLLImp void  SetCBRoboExtState(void (__stdcall *)(DWORD, DWORD));

//  get transfer statistics, info
DLLImp DWORD GetSessionId(void);
DLLImp DWORD GetTrapStatistic(HANDLE, DWORD *, DWORD *, DWORD *, DWORD *, DWORD *, DWORD *);
DLLImp void  ResetTrapStatistic(HANDLE);

//  firmware update
DLLImp DWORD RoboTxFwUpdate(HANDLE, LPCSTR, void (__stdcall *)(DWORD, LPSTR));

#endif