// -----------------------------------------------------------------------------------------
//
// umFish50.CPP                                                       Ulrich Mller 15.11.09
//                                                                    PCS1700       VC++ 6.0
// Basisfunktionen fr den fischertechnik ROBO TX Controller unter
//                 Nutzung der FtMscLib.DLL
// Hinweis : Die Universal-Eingnge werden fliegend - bei ihrer ersten Nutzung - konfiguriert.
//           Um einen sicheren Betrieb zu gewhrleisten, werden Sleeps eingestreut,
//           sie sind nach Bluetooth-Anforderungen ausgelegt.
// 
// ------------------------------------------------------------------------------------------

#include <Windows.h>
#include <stdio.h>
#include <iostream.h>
#include <mmsystem.h>

#include "umFish50.h"

#pragma   once
#define  _CRT_SECURE_NO_DEPRECATE
#define   WIN32_LEAN_AND_MEAN 
#include "Inc/targetver.h"
#include "Inc/common.h"
#include "Inc/ROBO_TX_FW.h"
#include "Inc/ftMscLib.h"
#include "Inc/ftErrCode.h"


const int txFehler       = 0xE0000001;
const int txPollInterval = 5;
const int txWarten       = 99;
const int modeVolt       = 0;
const int modeOhm        = 1;
const int modeDist       = 3;

HANDLE    fthdl;
DWORD     errCode;
MMRESULT  FID;
DWORD     Dummy;
bool      firstTime[9] [12];
int       zielListe[9] [4] [3];


// PollInterface -------------------------------------------------------------------
//
// Loop ber alle Counter-Eingnge : Bei Zhlerstand >= Sollstand Stop zugehrender
//      Motor, Rcksetzen des Solls auf 0, Setzen der Differenz zum Soll
//      Bei RobMode und Linkslauf auch Abbruch bei Erkennen des zug. Endtasters
//
// ---------------------------------------------------------------------------------

void CALLBACK PollInterface(UINT wTimerID, UINT msg, DWORD DCB, DWORD dw1, DWORD dw2) 
{
	int   ziel;
	int   robLinks;
	short count    = 0;
	short state    = 0;
	short wert     = 0;
	UINT  ok       = FALSE;

	int i, j;

	if(GetAsyncKeyState(0x1B) != 0) {
		for(i = 0; i < 9; i++)
			for(j = 0; j < 4; j++) SetOutMotorValues(fthdl, i, j, 0, 0, FALSE);
	}
	for (i = 0; i < 9; i++) {
		for (j = 0; j < 4; j++) {
			robLinks = zielListe[i][j][0];
			ziel = zielListe[i][j][1];
			if (ziel > 0) {
				GetInCounterValue(fthdl, i, j, &count, &state);
				if (count >= ziel) {
					SetOutCounterReset(fthdl, i, j);
          SetOutMotorValues(fthdl, i, j, 0, 0, FALSE);
					zielListe[i][j][0] = 0;
					zielListe[i][j][1] = 0;
					zielListe[i][j][2] = count - ziel;
				}
				else if (robLinks > 0) {
					wert = 0;
					GetInIOValue(fthdl, i, j, &wert, &ok);
					if (wert != 0) {
						SetOutCounterReset(fthdl, i, j);
						SetOutMotorValues(fthdl, i, j, 0, 0, FALSE);
					zielListe[i][j][0] = 0;
					zielListe[i][j][1] = 0;
					zielListe[i][j][2] = 0;
					}
				}
			}
		}
	}
}

// txOpenController ---------------------------------------------------------------
//
// Herstellen einer Verbindung zum TX Controller an USB oder Bluetooth
//    Starten des MultiMediaTimers
//
// --------------------------------------------------------------------------------

int __stdcall txOpenController(int ComNr) {
	fthdl = 0;
	int i, j;
	if(ftxInitLib() != FTLIB_ERR_SUCCESS) return txFehler;
  fthdl  = ftxOpenComDeviceNr(ComNr, 38400, &errCode);
	if(errCode != FTLIB_ERR_SUCCESS) return txFehler;
	if(ftxStartTransferArea(fthdl) != FTLIB_ERR_SUCCESS) return txFehler;
	for (i = 0; i < 9; i++) {
		for (j = 0; j < 12; j++) firstTime[i][j] = FALSE;
		for (j = 0; j < 4; j++) {
			zielListe[i][j][0] = 0;
			zielListe[i][j][1] = 0;
			zielListe[i][j][2] = 0;
		}
	}
	Sleep(txWarten * 5);
	FID = timeSetEvent(txPollInterval, 0, PollInterface, DWORD(&Dummy), TIME_PERIODIC);
	return 0;
}

// txCloseController ----------------------------------------------------------------
//
// Beenden der Verbindung zum TX Controller, Beenden MultiMediaTimer
//
// ----------------------------------------------------------------------------------

int __stdcall txCloseController() {
	if(fthdl == 0) return txFehler;
  Sleep(txWarten * 3);
	if(ftxStopTransferArea(fthdl) != FTLIB_ERR_SUCCESS) return txFehler;
	if(ftxCloseDevice(fthdl) != FTLIB_ERR_SUCCESS) return txFehler;
	if(ftxCloseLib() != FTLIB_ERR_SUCCESS) return txFehler;
	fthdl = 0;
	if(FID != 0) timeKillEvent(FID);
	Sleep(txWarten * 5);
	return 0;
}

// txGetAnalog ----------------------------------------------------------------------
//
// Abfrage I-Eingang auf Analogwerte (NTC, Photowiderstand, Potentiometer)
//
// ----------------------------------------------------------------------------------

int __stdcall txGetAnalog(int ctrId, int inpNr) {
	short wert = 0;
	UINT  ok = 0;
	if (fthdl == 0) return txFehler;
	if (!firstTime[ctrId][inpNr]) {
		if (SetFtUniConfig(fthdl, ctrId, inpNr, modeOhm, 0) != FTLIB_ERR_SUCCESS) 
			return txFehler;
		Sleep(txWarten);
		firstTime[ctrId][inpNr] = TRUE;
	}
	if (GetInIOValue(fthdl, ctrId, inpNr, &wert, &ok) != FTLIB_ERR_SUCCESS) 
		return txFehler;
	return wert;
}

// txGetDistance -----------------------------------------------------------------------
//
// Abfrage I-Eingang auf Distanzwerte (Ultraschallsensor)
//
// -------------------------------------------------------------------------------------

int __stdcall txGetDistance(int ctrId, int inpNr) {
	short wert = 0;
	UINT  ok = 0;
	if (fthdl == 0) return txFehler;
	if (!firstTime[ctrId][inpNr]) {
		if (SetFtUniConfig(fthdl, ctrId, inpNr, modeDist, 0) != FTLIB_ERR_SUCCESS) 
			return txFehler;
		Sleep(txWarten);
		firstTime[ctrId][inpNr] = TRUE;
	}
	if (GetInIOValue(fthdl, ctrId, inpNr, &wert, &ok) != FTLIB_ERR_SUCCESS) return txFehler;
	return wert;
}

// txGetInput -----------------------------------------------------------------------------
//
// Abfrage I-Eingang auf Wahrheitswerte (Taster, Reedkontakt, Phototransistor)
//         auch an den Eingngen C1 - C4
//
// ----------------------------------------------------------------------------------------

int __stdcall txGetInput(int ctrId, int inpNr) {
	short wert = 0;
	UINT   ok  = 0;
	short  okC = 0;
	if (fthdl == 0) return txFehler;
  if (inpNr < 8) {
  	if (!firstTime[ctrId][inpNr]) {
	  	if (SetFtUniConfig(fthdl, ctrId, inpNr, modeOhm, 1) != FTLIB_ERR_SUCCESS) 
				return txFehler;
		  Sleep(txWarten);
		  firstTime[ctrId][inpNr] = TRUE;
		}
  	if (GetInIOValue(fthdl, ctrId, inpNr, &wert, &ok) != FTLIB_ERR_SUCCESS) return txFehler;
		return wert == 0 ? FALSE : TRUE;
	} 
	else {
		if (!firstTime[ctrId][inpNr]) {
  		if (SetFtCntConfig(fthdl, ctrId, inpNr - 8, 1) != FTLIB_ERR_SUCCESS) return txFehler;
	  	Sleep(txWarten);
		  firstTime[ctrId][inpNr] = TRUE;
		}
		if (GetInCounterValue(fthdl, ctrId, inpNr - 8, &wert, &okC) != FTLIB_ERR_SUCCESS) 
			return txFehler;
		return ok;
	}
}

// txGetTrack -------------------------------------------------------------------------------
//
// Abfrage I-Eingang auf Wahrheitswert eines einzelnen Ausgangs des Spursensors
//         TRUE : auf schwarzer Spur
//
// ------------------------------------------------------------------------------------------

int __stdcall txGetTrack(int ctrId, int inpNr) {
	short wert = 0;
	UINT  ok = 0;
	if (fthdl == 0) return txFehler;
	if (!firstTime[ctrId][inpNr]) {
		if (SetFtUniConfig(fthdl, ctrId, inpNr, modeVolt, TRUE) != FTLIB_ERR_SUCCESS) 
			return txFehler;
		Sleep(txWarten);
		firstTime[ctrId][inpNr] = TRUE;
	}
	if (GetInIOValue(fthdl, ctrId, inpNr, &wert, &ok) != FTLIB_ERR_SUCCESS) return txFehler;
	return wert > 0 ? FALSE : TRUE;
}

// txGetVoltage -----------------------------------------------------------------------------
//
// Abfrage I-Eingang auf Spannungswert (Farbsensor)
//
// ------------------------------------------------------------------------------------------

int __stdcall txGetVoltage(int ctrId, int inpNr) {
	short wert = 0;
	UINT  ok   = 0;
	if (fthdl == 0) return txFehler;
	if (!firstTime[ctrId][inpNr]) {
		if (SetFtUniConfig(fthdl, ctrId, inpNr, modeVolt, FALSE) != FTLIB_ERR_SUCCESS) 
			return txFehler;
		Sleep(txWarten);
		firstTime[ctrId][inpNr] = TRUE;
	}
	if (GetInIOValue(fthdl, ctrId, inpNr, &wert, &ok) != FTLIB_ERR_SUCCESS) return txFehler;
	return wert;
}


// txGetCounter ----------------------------------------------------------------------------
//
// Abfrage des aktuellen Counterwertes an C1 - C4
//
// -----------------------------------------------------------------------------------------

int __stdcall txGetCounter(int ctrId, int cntNr) {
	short wert = 0;
	short okC  = 0;
	if (fthdl == 0) return txFehler;
  if (GetInCounterValue(fthdl, ctrId, cntNr, &wert, &okC) != FTLIB_ERR_SUCCESS) 
		return txFehler;
	return wert;
}

// txClearCounter --------------------------------------------------------------------------
//
// Zurcksetzen auf 0 eines Counters an C1 - C4
//
// -----------------------------------------------------------------------------------------

int __stdcall txClearCounter(int ctrId, int cntNr) {
	if (fthdl == 0) return txFehler;
	if (SetOutCounterReset(fthdl, ctrId, cntNr) != FTLIB_ERR_SUCCESS) return txFehler;
	Sleep(txPollInterval*2);
	return 0;
}


// txSetLamp ------------------------------------------------------------------------------
//
// Setzen eines O-Ausganges auf die vorgegebene Intensitt (0 = aus)
//
// ----------------------------------------------------------------------------------------

int __stdcall txSetLamp(int ctrId, int outNr, int OnOff) {
	if(fthdl == 0) return txFehler;
	errCode = SetOutPwmValues(fthdl, ctrId, outNr, OnOff == 0 ? 0 : OnOff);
  if(errCode != FTLIB_ERR_SUCCESS) return txFehler;
	return 0;
}

// txStartMotor -----------------------------------------------------------------------------
//
// Einschalten eines Motors am M-Ausganges mit whlbarer Richtung und Intensitt
//    Zustzlich wird ein ImpulsCount ausgewertet, der Motor wird bei
//    Erreichen vom PollInterface abgeschaltet (EncoderMotoren, normale + Impulstaster). 
//    Im RobMode wird der Motor im LinksLauf auch bei Erreichen eines Endtaster abgeschaltet
//
// ------------------------------------------------------------------------------------------

int __stdcall txStartMotor(int ctrId, int outNr, int dir, int speed, int iCount, int robMode)
{
	int links, rechts;
	if (fthdl == 0) return txFehler;
	switch(dir) {
		case 1:
			links = speed;
			rechts = 0;
			break;
		case 2:
			links = 0;
			rechts = speed;
			break;
		default:
			links = 0;
			rechts = 0;
			break;
	}
	if (iCount > 0) {
	  if (SetOutCounterReset(fthdl, ctrId, outNr) != FTLIB_ERR_SUCCESS) return txFehler;
	  Sleep(txWarten);
	}
	zielListe[ctrId][outNr][0] = robMode == 1 && dir == 1 ? 1 : 0;
  zielListe[ctrId][outNr][1] = iCount;
	zielListe[ctrId][outNr][2] = 0;
	if (SetOutMotorValues(fthdl, ctrId, outNr, links, rechts, FALSE) != FTLIB_ERR_SUCCESS) 
		return txFehler;
	return 0;
}

// txSetMotor --------------------------------------------------------------------------------
//
// Schalten eines M-Ausganges mit whlbarer Richtung und Intensitt
//
// -------------------------------------------------------------------------------------------

int __stdcall txSetMotor(int ctrId, int outNr, int dir, int speed) {
	txStartMotor(ctrId, outNr, dir, speed, 0, 0);
	return 0;
}

// txIsMotorReady ----------------------------------------------------------------------------
//
// Rckgabe eines Ready-Wertes fr einen ber StartMotor geschalteten Motors
//    -1 : luft noch, 0 und grer : abgeschaltet, Angabe der Abweichung von der Zielvorgabe
//
// -------------------------------------------------------------------------------------------

int __stdcall txIsMotorReady(int ctrId, int outNr) {
	if (fthdl == 0) return txFehler;
	Sleep(txWarten);
	if (zielListe[ctrId][outNr][1] > 0) return -1;
	return zielListe[ctrId][outNr][2];
}

// Win32 Service Funktionen -----------------------------------------------------------------
//
// Sleep, Esc-Taste, TickCount
//
// ------------------------------------------------------------------------------------------

void __stdcall txSleep(int mSek) {Sleep(mSek);}
int __stdcall txEsc() {return GetAsyncKeyState(VK_ESCAPE);}
int __stdcall txGetTickCount() {return GetTickCount();}

// DLLMain ----------------------------------------------------------------------------------
//
// Entladen der DLL
//
// -----------------------------------------------------------------------------------------

BOOL WINAPI DllMain(HINSTANCE hDLL, DWORD dwReason, LPVOID){
	switch(dwReason){
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}
